package fr.mael.jiwigo.ui.mainframe;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;

import fr.mael.jiwigo.transverse.enumeration.PreferencesEnum;
import fr.mael.jiwigo.transverse.util.Messages;
import fr.mael.jiwigo.transverse.util.preferences.PreferencesManagement;

/**
 * 
   Copyright (c) 2010, Mael
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of jiwigo nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
   DISCLAIMED. IN NO EVENT SHALL Mael BE LIABLE FOR ANY
   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   
 * @author mael
 *
 */
public class PreferencesDialog extends JDialog implements ActionListener {

    /**
     * Logger
     */
    public static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory
	    .getLog(PreferencesDialog.class);

    /**
     * the tabs
     */
    private JTabbedPane tabPane;

    /**
     * preferences of the images tab
     */
    private JPanel panelImage;

    /**
     * width of the images
     */
    private JTextField fieldWidth;

    /**
     * height of the images
     */
    private JTextField fieldHeight;

    /**
     * size of the chunks
     */
    private JTextField fieldChunkSize;

    /**
     * label
     */
    private JLabel labelTaille;

    /**
     * label
     */
    private JLabel labelChunkSize;

    /**
     * label
     */
    private JLabel labelCroix;

    /**
     * save the preferences
     */
    private JButton boutonOk = new JButton("Ok");

    /**
     * panel that contains the buttons
     */
    private JPanel panelBoutons;

    /**
     * bouton to cancel 
     */
    private JButton boutonAnnuler;

    /**
     * Constructor
     * @param parent frame parent
     */
    public PreferencesDialog(JFrame parent) {
	fieldWidth = new JTextField(PreferencesManagement.getValue(PreferencesEnum.WIDTH_ORIGINALE.getLabel()));
	fieldHeight = new JTextField(PreferencesManagement.getValue(PreferencesEnum.HEIGHT_ORIGINAL.getLabel()));
	fieldChunkSize = new JTextField(PreferencesManagement.getValue(PreferencesEnum.CHUNK_SIZE.getLabel()));
	labelTaille = new JLabel(Messages.getMessage("preferences_imagesSize"));
	labelCroix = new JLabel("x");
	labelChunkSize = new JLabel(Messages.getMessage("preferences_chunkSize"));
	fieldWidth.setPreferredSize(new Dimension(50, 30));
	fieldHeight.setPreferredSize(new Dimension(50, 30));
	fieldChunkSize.setPreferredSize(new Dimension(50, 30));

	panelImage = new JPanel(new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0;
	c.gridy = 0;
	c.anchor = GridBagConstraints.WEST;
	panelImage.add(labelTaille, c);
	c.gridx++;
	panelImage.add(fieldWidth, c);
	c.gridx++;
	panelImage.add(labelCroix, c);
	c.gridx++;
	panelImage.add(fieldHeight, c);
	c.gridy++;
	c.gridx = 0;
	panelImage.add(labelChunkSize, c);
	c.gridx++;
	panelImage.add(fieldChunkSize, c);

	boutonAnnuler = new JButton(Messages.getMessage("cancel"));
	boutonAnnuler.addActionListener(this);
	boutonOk.addActionListener(this);
	panelBoutons = new JPanel(new FlowLayout(FlowLayout.RIGHT));
	panelBoutons.add(boutonOk);
	panelBoutons.add(boutonAnnuler);
	boutonOk.setPreferredSize(new Dimension(90, 30));
	boutonAnnuler.setPreferredSize(new Dimension(90, 30));

	tabPane = new JTabbedPane();
	tabPane.addTab(Messages.getMessage("preferences_tabImage"), panelImage);
	this.setLayout(new BorderLayout());
	this.add(tabPane, BorderLayout.CENTER);
	this.add(panelBoutons, BorderLayout.SOUTH);
	this.setSize(new Dimension(400, 300));
	this.setLocationRelativeTo(null);
	this.setVisible(true);
    }

    @Override
    public void actionPerformed(ActionEvent e) {
	if (e.getSource().equals(boutonAnnuler)) {
	    this.dispose();
	} else if (e.getSource().equals(boutonOk)) {
	    try {
		int width = Integer.valueOf(fieldWidth.getText());
		int height = Integer.valueOf(fieldHeight.getText());
		double chunk = Double.parseDouble(fieldChunkSize.getText().replace(",", "."));
		PreferencesManagement.setValue(PreferencesEnum.WIDTH_ORIGINALE.getLabel(), String.valueOf(width));
		PreferencesManagement.setValue(PreferencesEnum.HEIGHT_ORIGINAL.getLabel(), String.valueOf(height));
		PreferencesManagement.setValue(PreferencesEnum.CHUNK_SIZE.getLabel(), String.valueOf(chunk));
		this.dispose();
	    } catch (NumberFormatException ex) {
		ex.printStackTrace();
		JOptionPane.showMessageDialog(this, Messages.getMessage("preferencesCheckValues"), Messages
			.getMessage("error"), JOptionPane.ERROR_MESSAGE);
	    } catch (Exception ex) {
		JOptionPane.showMessageDialog(this, Messages.getMessage("preferencesUnexpectedError"), Messages
			.getMessage("error"), JOptionPane.ERROR_MESSAGE);

	    }

	}

    }
}
